#!/bin/bash
# Copyright © 2020 symphonic-mpd Research and Development Club

LGPATH="/var/lib/mpd/music/RAM"
LOG="${LGPATH}/.radiko.out"
LOG2="${LGPATH}/.rarea.out"
LOG3="${LGPATH}/.rinfo.out"
LOG4="${LGPATH}/.rprog.out"

get_top() {
  YTTOP=`ps -ef | grep "yt-dlp" | grep -v grep | head -n 1 | awk '{ print $2 }'`
  RATOP=`ps -ef | grep "ffplay" | grep -v grep | awk '{ print $2 }'`
}

stop_play() {
  if [ "${YTTOP}" != "" ]; then
    kill ${YTTOP}
  elif [ "${RATOP}" != "" ]; then
    kill ${RATOP}
  fi
}

remove_log() {
  if [ -f ${LOG} ]; then
    echo "" > ${LOG}
  else
    touch ${LOG}
  fi
}

PARAM=$1
get_top

case "$PARAM" in
  startup )
    if [ "${YTTOP}" == "" ] && [ "${RATOP}" == "" ]; then
      remove_log
    fi
  ;;
  rarea )
    curl "https://radiko.jp/area" -s -o ${LOG2}
  ;;
  rinfo* )
    ARR=${PARAM/rinfo/}
    curl "https://radiko.jp/v3/program/now/${ARR}.xml" -s -o ${LOG3}
  ;;
  rprog* )
    ARR=${PARAM/rprog/}
    RTM=`date +'%H'`
    if [ $RTM -lt 5 ]; then
      RTM2=`date +%Y%m%d --date '1 day ago'`
    else
      RTM2=`date +%Y%m%d`
    fi
    curl "https://radiko.jp/v3/program/station/date/${RTM2}/${ARR}.xml" -s -o ${LOG4}
  ;;
  radiko* )
    ARR=${PARAM//,/ }
    ARR=${ARR/radiko/}
    PWD=$(cd $(dirname $0); pwd)
    stop_play
    remove_log
    nohup ${PWD}/radish-play.sh ${ARR} > ${LOG} 2>&1 &
    wait $!
  ;;
  youtube* )
    ARR=${PARAM/youtube/}
    stop_play
    remove_log
    yt-dlp_linux_aarch64 -f ba/ba* --no-continue -x -o - "${ARR}" 2>${LOG} | ffplay -i - -vn -nodisp -autoexit -loglevel error -fflags +discardcorrupt >> ${LOG} 2>&1 &
  ;;
  stop )
    stop_play
    remove_log
  ;;
esac

exit
