#!/bin/sh

PLUGIN=/opt/plugins
VARLIB=/var/lib

# install ympd main application
function install_ympd() {
    echo "Installing ympd config files"

    if [ ! -d $VARLIB ]; then
      mkdir $VARLIB
    fi
    if [ ! -d $VARLIB/mpd ]; then
      mkdir $VARLIB/mpd
    fi
    if [ ! -d $VARLIB/mpd/music ]; then
      mkdir $VARLIB/mpd/music
    fi
    if [ ! -d $VARLIB/mpd/music/NAS ]; then
      mkdir $VARLIB/mpd/music/NAS
    fi
    if [ ! -d $VARLIB/mpd/music/RAM ]; then
      mkdir $VARLIB/mpd/music/RAM
    fi
    if [ ! -d $VARLIB/mpd/music/RADIO ]; then
      mkdir $VARLIB/mpd/music/RADIO
    fi
    if [ ! -d $VARLIB/mpd/music/assets ]; then
      mkdir $VARLIB/mpd/music/assets
    fi

    cp NAS.jpg $VARLIB/mpd/music
    cp RADIO.jpg $VARLIB/mpd/music
    cp RAM.jpg $VARLIB/mpd/music

    cp webui.cfg.org /etc

    cp ympd-plus.sh /usr/bin
    cp edit_fstab.sh /usr/local/bin
    cp ympd_event.sh /usr/local/bin
    chmod 755 /usr/bin/ympd-plus.sh
    chmod 755 /usr/local/bin/edit_fstab.sh
    chmod 755 /usr/local/bin/ympd_event.sh
}

# install plugins
function install_plugins() {
    echo "Installing ympd plugins"
    dirary=("01-soundcard" "02-net" "03-search" "04-webui" "05-stats" "06-no_image" "10-mpd" "11-update_library" "12-save_queue" "13-add_stream"  "14-upload"  "15-lastfm" "20-color" "40-spotify" "81-disk_usage" "82-aoe"  "83-key_help" "84-radiko_youtube" "90-plugins" "91-support"  "92-reboot" "93-shutdown") 

    if [ ! -d $PLUGIN ]; then
      mkdir $PLUGIN
    fi
    if [ ! -d $PLUGIN/attach ]; then
      mkdir $PLUGIN/attach
    fi
    if [ ! -d $PLUGIN/detach ]; then
      mkdir $PLUGIN/detach
    fi

    for str in ${dirary[@]}; do
       if [ ! -d $PLUGIN/$str ]; then
           mkdir $PLUGIN/$str
       fi
       cp $str/* $PLUGIN/$str
       if [ -e $PLUGIN/$str/run.sh ]; then
           chmod 755 $PLUGIN/$str/run.sh
       fi
    done

    pastsdir=`pwd`

    cd $PLUGIN/attach
    ln -s ../90-plugins 90-plugins
    cd $pastdir

    cd /var/lib/mpd/music
    ln -s /opt/plugins plugins
    cd $pastdir
    chmod 755 $PLUGIN/84-radiko_youtube/radish-play.sh
}

#find non-symlink directory and make symlink
function link_make() {
    DIRS=`find $PLUGIN -maxdepth 1 -type d`

    pastdir=`pwd`
    cd $PLUGIN/detach

    for dir in $DIRS;
    do
        dir=`basename $dir`
        if [ $dir != "attach" ] && [ $dir != "detach" ] && [ $dir != "plugins" ] && [ $dir != "90-plugins" ]; then
          if [ ! -L $PLUGIN/attach/$dir ] && [ ! -L $PLUGIN/detach/$dir ]; then
            echo "making symlink file of directroy $dir in detach dir"
            ln -s ../$dir $dir
          fi
          if [ -L $PLUGIN/attach/$dir ] && [ -L $PLUGIN/detach/$dir ]; then
            echo "delete symlink file fo directory $dir in detach dir because there are symlink files both detach and attach dir"
            rm $PLUGIN/detach/$dir
          fi
        fi
    done
    cd $pastdir
}

function usage() {
    echo "install.sh arg"
    echo "  arg) ympd: install ympd program and restart"
    echo "       plugin: install ympd plug-in"
    echo "       link: check plugin symkinks in /opt/plugin and make it in detach directry (if not found)"
    echo "       (none): do all installation"
}

case "$1" in
    "help")
        usage
        exit 0
        ;;
    "ympd")
        install_ympd
        ;;
    "plugin")
        install_plugins
        ;;
    "link")
        link_make
        ;;
    *)
        install_ympd
        install_plugins
        link_make
esac

# do process after install
sync

echo "done!"

exit 0
